@@ -11,7 +11,11 @@ public:
 
     MY_UNKNOWN_IMP1(ICompressCodecsInfo)
 
+#if MY_VER_MAJOR >= 15
+    STDMETHOD(GetNumMethods)(UInt32 *numMethods);
+#else    
     STDMETHOD(GetNumberOfMethods)(UInt32 *numMethods);
+#endif	
     STDMETHOD(GetProperty)(UInt32 index, PROPID propID, PROPVARIANT *value);
     STDMETHOD(CreateDecoder)(UInt32 index, const GUID *interfaceID, void **coder);
     STDMETHOD(CreateEncoder)(UInt32 index, const GUID *interfaceID, void **coder);
