@@ -4,6 +4,12 @@
 #undef S_OK
 #endif
 
+#if !defined(_WIN32) && !defined(_OS2)
+#include "CPP/myWindows/StdAfx.h"
+#include "CPP/include_windows/windows.h"
+#endif
+
+#include "C/7zVersion.h"
 #include "CPP/7zip/Archive/IArchive.h"
 #include "CPP/Windows/PropVariant.h"
 #include "CPP/Common/MyCom.h"
