@@ -8,6 +8,7 @@
 #undef S_OK
 #endif
 
+#define BUILD_LIB7ZIP
 #ifndef INITGUID
 #define INITGUID
 #endif
@@ -18,6 +19,7 @@
 #include "CPP/7zip/MyVersion.h"
 #endif
 
+#include "C/7zVersion.h"
 #include "CPP/7zip/Archive/IArchive.h"
 #include "CPP/Windows/PropVariant.h"
 #include "CPP/Common/MyCom.h"
@@ -145,16 +147,19 @@ HRESULT GetMethodPropertyGUID(GetMethodPropertyFunc getMethodProperty, UInt32 in
     return S_OK;
 }
 
-
+#if MY_VER_MAJOR >= 15
+//after version 15, MyCharUpper defined inline in MyString.h
+#else
 #ifdef _WIN32
-inline wchar_t MyCharUpper(wchar_t c)
+inline wchar_t MyCharUpper(wchar_t c) LIB7ZIP_THROW
 { return (wchar_t)(unsigned int)(UINT_PTR)CharUpperW((LPWSTR)(UINT_PTR)(unsigned int)c); }
 #else
-inline wchar_t MyCharUpper(wchar_t c)
+inline wchar_t MyCharUpper(wchar_t c) LIB7ZIP_THROW
 { return toupper(c); }
 #endif
+#endif
 
-int MyStringCompareNoCase(const wchar_t *s1, const wchar_t *s2)
+int MyStringCompareNoCase(const wchar_t *s1, const wchar_t *s2) LIB7ZIP_THROW
 {
 	for (;;) {
 		wchar_t c1 = *s1++;
@@ -284,11 +289,17 @@ wstring WidenString( const string& str )
 {
 	std::wostringstream wstm ;
     const char * loc = 
-        g_lib7zip_loc == NULL ? setlocale(LC_CTYPE, NULL) : g_lib7zip_loc;
+        g_lib7zip_loc == NULL ? setlocale(LC_CTYPE, "") : g_lib7zip_loc;
 
     if (loc == NULL || strlen(loc) == 0)
       loc = "C";
-	wstm.imbue(std::locale(loc));
+
+    try {
+      wstm.imbue(std::locale(loc));
+    }
+    catch(...) {
+      wstm.imbue(std::locale("C"));
+    }
 
 	const std::ctype<wchar_t>& ctfacet =
 		 std::use_facet< std::ctype<wchar_t> >( wstm.getloc() ) ;
@@ -316,5 +327,8 @@ const char * GetLib7ZipLocale()
 
 const char * SetLib7ZipLocale(const char * loc)
 {
+  const char * tmp = g_lib7zip_loc;
+
   g_lib7zip_loc = loc;
+  return tmp;
 }
