@@ -5,6 +5,12 @@
 #undef S_OK
 #endif
 
+#if !defined(_WIN32) && !defined(_OS2)
+#include "CPP/myWindows/StdAfx.h"
+#include "CPP/include_windows/windows.h"
+#endif
+
+#include "C/7zVersion.h"
 #include "CPP/7zip/Archive/IArchive.h"
 #include "CPP/Windows/PropVariant.h"
 #include "CPP/Common/MyCom.h"
@@ -26,11 +32,16 @@ using namespace NWindows;
 #endif
 #include "dlfcn.h"
 #include "unistd.h"
+
+#ifdef __APPLE__
+#include <sys/types.h>
+#endif
+
 #include "dirent.h"
 
 #include "OSFunctions_UnixLike.h"
 
-#ifdef __APPLE__
+#if __APPLE__ && (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_9)
 int myselect(struct dirent * pDir );
 #else
 int myselect(const struct dirent * pDir );
@@ -67,7 +78,7 @@ bool LoadDllFromFolder(C7ZipDllHandler * pMainHandler, const wstring & wfolder_n
   return true;
 }
 
-#ifdef __APPLE__
+#if __APPLE__ && (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_9)
 int myselect(struct dirent * pDir )
 #else
 int myselect(const struct dirent * pDir )
