@@ -1,5 +1,7 @@
 #!/bin/sh
 libtoolize --force --copy
+#for macos
+glibtoolize --force --copy
 aclocal
 automake -a
 autoconf
